﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;



namespace CStestProject
{
    class Program
    {
        public struct CAN_MSG
        {
            public UInt32 Id;
            public byte Size;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 8)]
            public byte[] Data;
            public byte Flags;
            public UInt16 TimeStamp;
        }
        public const Int32 Normal = 0;
        public const Int32 ListenOnly = 1;
        public const Int32 LoopBack = 2;

        public const Int32 CAN_TIMESTAMP_OFF = 0;
        public const Int32 CAN_TIMESTAMP_ON = 1;

        public const byte CAN_FLAGS_STANDARD = (1 << 0);
        public const byte CAN_FLAGS_EXTENDED = (1 << 1);
        public const byte CAN_FLAGS_REMOTE = (1 << 2);
        public const byte CAN_FLAGS_TIMESTAMP = (1 << 3);

        public const Int32 CAN_STATUS_BOFF = 0x80;
        public const Int32 CAN_STATUS_EWARN = 0x40;
        public const Int32 CAN_STATUS_EPASS = 0x20;
        public const Int32 CAN_STATUS_LEC_NO = 0x00;
        public const Int32 CAN_STATUS_LEC_STUFF_ERR = 0x01;
        public const Int32 CAN_STATUS_LEC_FORM_ERR = 0x02;
        public const Int32 CAN_STATUS_LEC_ACK_ERR = 0x03;
        public const Int32 CAN_STATUS_LEC_BIT1_ERR = 0x04;
        public const Int32 CAN_STATUS_LEC_BIT0_ERR = 0x05;
        public const Int32 CAN_STATUS_LEC_CRC_ERR = 0x06;

        

        [DllImport("can_api.dll", EntryPoint = "CAN_Open", CallingConvention = CallingConvention.Cdecl)]
        static extern Int32 CAN_Open(string SerialNrORComPortORNet, string szBitrate, string acceptance_code, string acceptance_mask, Int32 flags, UInt32 Mode);

        [DllImport("can_api.dll", EntryPoint = "CAN_Write", CallingConvention = CallingConvention.Cdecl)]
        static extern Int32 CAN_Write(Int32 Handle, CAN_MSG[] Buf);

        [DllImport("can_api.dll", EntryPoint = "CAN_Read", CallingConvention = CallingConvention.Cdecl)]
        static extern Int32 CAN_Read(Int32 Handle, ref CAN_MSG Buf);

        [DllImport("can_api.dll", EntryPoint = "CAN_Flush", CallingConvention = CallingConvention.Cdecl)]
        static extern Int32 CAN_Flush(Int32 Handle);

        [DllImport("can_api.dll", EntryPoint = "CAN_Close", CallingConvention = CallingConvention.Cdecl)]
        static extern Int32 CAN_Close(Int32 Handle);

        [DllImport("can_api.dll", EntryPoint = "CAN_Status", CallingConvention = CallingConvention.Cdecl)]
        static extern Int32 CAN_Status(Int32 Handle);

        [DllImport("can_api.dll", EntryPoint = "CAN_Version", CallingConvention = CallingConvention.Cdecl)]
        static extern Int32 CAN_Version(Int32 Handle, StringBuilder Buf);


        static void Main(string[] args)
        {
            Console.Write("Port Name:");
            string szBitrate = "1000";
            string acceptance_code = "00000000";
            string acceptance_mask = "00000000";
            string ComPort = Console.ReadLine();
            uint mode = LoopBack;
            int handle=-1, Status=0, flag = CAN_TIMESTAMP_OFF;
            StringBuilder FWVER = new StringBuilder();
            CAN_MSG[] SendMSG = new CAN_MSG[1];
            CAN_MSG[] RecvMSG = new CAN_MSG[1];
            RecvMSG[0].Data = new byte[8];
            SendMSG[0].Data = new byte[8];

            SendMSG[0].Flags = CAN_FLAGS_EXTENDED;
            SendMSG[0].Id = 0x12345678;
            SendMSG[0].Size = 8;
            SendMSG[0].Data[0] = 0x11;
            SendMSG[0].Data[1] = 0x22;
            SendMSG[0].Data[2] = 0x33;
            SendMSG[0].Data[3] = 0x44;
            SendMSG[0].Data[4] = 0x55;
            SendMSG[0].Data[5] = 0x66;
            SendMSG[0].Data[6] = 0x77;
            SendMSG[0].Data[7] = 0x88;


            handle = CAN_Open(ComPort, szBitrate, acceptance_code, acceptance_mask, flag, mode);
            Console.Write("handle = {0}\n",handle);
            if (handle < 0)
            {
                return;
            }
            Status = CAN_Flush(handle);

            CAN_Version(handle, FWVER);
            FWVER.Replace("\r", "\0");
            FWVER.Replace("\n", "\0");
            
            Console.Write("Firmware:");
            int i = 0;
            while ((FWVER[i] != '\0') && i < FWVER.Length)
            {
                Console.Write("{0}", FWVER[i]);
                i++;
            }
            Console.Write("\n");



            Status = CAN_Write(handle, SendMSG);
            if (Status == 1)
            {
                Console.Write("Write Success\n");
                Status = 0;
            }

            while (true)
            {
                Status = CAN_Read(handle, ref RecvMSG[0]);
                if (Status == 1)
                {
                    int a = RecvMSG[0].Flags & CAN_FLAGS_STANDARD;
                    Console.Write("Read ID={0:X}, Type={1}, DLC={2}, FrameType={3}, Data=",
                        RecvMSG[0].Id,
                        ((RecvMSG[0].Flags & CAN_FLAGS_STANDARD)==1) ? "STD" : "EXT",
                        RecvMSG[0].Size,
                        ((RecvMSG[0].Flags & CAN_FLAGS_REMOTE)==1) ? "REMOTE" : "DATA");

                    for (i = 0; i < RecvMSG[0].Size; i++)
                    {
                        Console.Write("{0:X}", RecvMSG[0].Data[i] );
                    }
                    Console.Write("\n");
                    Status = 0;
                    break;
                }
            }

            Status = CAN_Close(handle);
            Console.Write("Test finish\n");
            Console.Read();

        }
    }
}
