###############################################################################
# File name: TITAN CAN Python Sample.py: Sample code for using CAN API
# Author: Frank Ying
# Copyright: Copyright 2017, Titan Electronics Inc.
# Version: 1.0.0
###############################################################################

# IMPORTS
from os import path
from ctypes import windll, Structure, byref, c_uint, c_ubyte, c_ushort, c_char_p

# Gets the current path and assumes CANDLL_STDCALL.dll is in the same folder
DLLPath = path.dirname(path.realpath(__file__))
DLLPath = DLLPath + "\CANDLL_STDCALL.dll"
# Load DLL
DLL = windll.LoadLibrary(DLLPath)

###############################################################################

# CAN_MSG STRUCTURE
""" C structure with C data types """
class CAN_MSG(Structure):
    _fields_ =  [("Id", c_uint),
                 ("Size", c_ubyte),
                 ("Data", c_ubyte * 8 ),
                 ("Flags", c_ubyte),
                 ("Timestamp", c_ushort)]

    ## Constructor
    """ Give default values to a newly created CAN_MSG instance *Optional """
    def __init__(self, Id = 0, Size = 0, Data = (0, 0, 0, 0, 0, 0, 0, 0), Flags = 0, Timestamp = 0):
        super(CAN_MSG, self).__init__(Id, Size, Data, Flags, Timestamp)

    ## String representation function
    """ Changes behavior of print(CAN_MSG) *Optional """
    def __str__(self):
        return ("ID: {}, Size: {}, Data: [{}, {}, {}, {}, {}, {}, {}, {}], Flags: {}, Timestamp: {}".format(
            format(self.Id, 'X'),
            format(self.Size, 'X'),
            format(self.Data[0], 'X'),
            format(self.Data[1], 'X'),
            format(self.Data[2], 'X'),
            format(self.Data[3], 'X'),
            format(self.Data[4], 'X'),
            format(self.Data[5], 'X'),
            format(self.Data[6], 'X'),
            format(self.Data[7], 'X'),
            format(self.Flags),
            format(self.Timestamp, 'X')))

###############################################################################

# Welcome message
print("Welcome to TITAN CAN Python Sample Program. We hope that this sample\
 code will allow you to further understand the way to use our TITAN CAN DLL to\
 create your own programs.\n")

# Prompts user for COM port to open
comPort = raw_input("Enter COM Port number (e.g. COM3, COM4...etc.): ")
# Converts user input to correct format
comPort = c_char_p(bytes(comPort))

# Test CAN_Open
raw_input("Testing CAN_Open... Press enter to continue.\n")
myHandle = DLL.CAN_Open(comPort, b"1000", b"00000000", b"00000000", 1, 2)
if (myHandle > 0):
    print("CAN_Open successful with handle {}!".format(myHandle))
elif (myHandle == -2):
    print("Error opening channel! Check if the COM port number is correct.")
    myHandle = 0
elif (myHandle == -1):
    print("Error communicating with device!")
    myHandle = 0

# Test CAN_Write
raw_input("Testing CAN_Write... Press enter to continue.\n")
# Create message to write
myWriteCANMSG = CAN_MSG()
myWriteCANMSG.Id = 0x1FFFFFFF
myWriteCANMSG.Size = 8
for i in xrange(8):
    myWriteCANMSG.Data[i] = i
myWriteCANMSG.Flags = 10
ret = DLL.CAN_Write(myHandle, byref(myWriteCANMSG))
if (ret == 1):
    print("CAN_Write successful!")
    print(myWriteCANMSG)
elif(ret == -4):
    print("COM port is not open!")
elif(ret == -1):
    print("Error communicating with device!")

# Test CAN_Read
raw_input("Testing CAN_Read... Press enter to continue.\n")
myReadCANMSG = CAN_MSG()
DLL.CAN_Read(myHandle, byref(myReadCANMSG))
if (ret == 1):
    print("CAN_Read successful!")
    print(myReadCANMSG)
elif (ret == -5):
    print("Message is empty!")
elif (ret == -4):
    print("COM port is not open!")
elif (ret == -1):
    print("Error communicating with device!")

# Test CAN_Close
raw_input("Testing CAN_Close... Press enter to continue.\n")
ret = DLL.CAN_Close(myHandle)
if (ret == 1):
    print("CAN_Close successful!")
elif(ret == -4):
    print("COM port is not open!")
elif(ret == -1):
    print("Error communicating with device!")

raw_input("Test complete! Press enter to exit.\n")
